@@ -4,50 +4,25 @@ use 5.008;
 use strict;
 use warnings;
 no warnings qw(uninitialized);
-use Carp;
-use File::Spec;
 
-our $VERSION = '0.024';
+our $VERSION = '0.030';
 $VERSION = eval $VERSION;
 
 sub import {
     my $package = shift;
     for my $category (@_) {
-        if (!ref($category)) {
-            enable_warning_category($category);
-        }
+        enable_warning_category($category);
     }
 }
 
 sub unimport {
     my $package = shift;
-    for my $args (@_) {
-        if (ref($args)) {
-            $package->_check_import_argument($args);
-            $package->_thwart_modules(%$args);
-        } else {
-            disable_warning_category($args);
-        }
+    for my $category (@_) {
+        disable_warning_category($category);
     }
 }
 
-sub _check_import_argument {
-    my ($package, $argument) = @_;
-
-    return if !ref($argument);
-    if (ref($argument) ne 'HASH') {
-        croak "Unexpected import argument $argument";
-    }
-    if (!exists $argument->{warning} || !exists $argument->{thwart_module}) {
-        croak "Argument keys must include warning and thwart_module";
-    }
-    if (   ref($argument->{thwart_module})
-        && ref($argument->{thwart_module}) ne 'ARRAY')
-    {
-        croak "Argument thwart_module should be a scalar or an arrayref";
-    }
-    _check_warning_category($argument->{warning}) or die;
-}
+use Carp;
 
 =head1 NAME
 
@@ -55,11 +30,10 @@ warnings::everywhere - a way of ensuring consistent global warning settings
 
 =head1 VERSION
 
-This is version 0.022.
+This is version 0.030.
 
 =head1 SYNOPSIS
 
- # Turn off run-time warnings
  use strict;
  use warnings;
  no warnings::anywhere qw(uninitialized);
@@ -70,22 +44,6 @@ This is version 0.022.
  use warnings::everywhere qw(uninitialized);
  # Write your own bondage-and-discipline code that really, really
  # cares about the difference between undef and the empty string
- 
- # Stop "helpful" modules from turning compile-time warnings back on again
- use strict;
- use warnings;
- no warnings::anywhere {
-     warning       => 'experimental::smartmatch',
-     thwart_module => [qw(Moose Moo Dancer Dancer2 Test::Class::Moose)],
- };
- use Module::That::Might::Pull::In::Moose::Or::Moo::Or::Who::Knows::What;
- 
- given (shift @ARGV) {
-     ...
-     default {
-         print STDERR "# I'll fix it in a moment, OK?\n";
-     }
- }
 
 =head1 DESCRIPTION
 
@@ -119,18 +77,10 @@ they were defined first.
 Either way, this isn't my code, and it's not something I'm going to fix.
 These warnings are just spam.
 
-Similarly, if you disable e.g. experimental::smartmatch because you know that
-you're using smartmatch, and you're not going to be using a version of
-Perl that has a version of smartmatch that behaves differently, you might
-get those warnings enabled back again by a module such as Moose or Dancer
-which turns all warnings on.
-
 This is where warnings::everywhere comes in.
 
 =head2 Usage
 
-=head3 Run-time warnings
-
 At its simplest, say
 
  use warnings::everywhere qw(all);
@@ -153,27 +103,6 @@ or
 Parameters are the same as C<use warnings>: a list of categories
 as per L<perllexwarn>, where C<all> means all warnings.
 
-=head3 Compile-time warnings
-
-This won't work for some (all?) compile-time warnings that are not just
-enabled for the module in question, but are injected back into your package.
-Moose, Moo, Dancer, Dancer2 and Test::Class::Moose all do this at the time of
-writing, by saying C<<warnings->import>> in their import method, thus
-injecting all warnings into I<your> package.
-
-To stop such code from turning back on warnings that you thought you'd
-disabled, say e.g.
-
- no warnings::anywhere {
-     warning       => 'experimental::smartmatch',
-     thwart_module => [qw(Moose)],
- };
-
-B<Warning>: warnings::everywhere disables these warnings by what is basically
-a source filter, so use with caution. If you can find an approved way of
-preventing modules such as Moose from doing this, do that rather than
-messing about with the module's source code!
-
 =head2 Limitations
 
 warnings::everywhere works by fiddling with the contents of the global hashes
@@ -205,7 +134,7 @@ perl gets to your use statement, so it's ignored.
 =item It's vulnerable to anything that sets $^W
 
 Any code that sets the global variable $^W, rather than saying C<use warnings>
-or C<<warnings->import>>, will turn on all warnings everywhere, bypassing the
+or C<warnings->import>, will turn on all warnings everywhere, bypassing the
 changes warnings::everywhere makes. This also includes any code that sets -w
 via the shebang.
 
@@ -237,12 +166,29 @@ after all. So you can't say
 and expect to get a warning from the last line. That warning's been
 turned off for good.
 
-=item Its method of disabling compile-time warnings is frankly iffy
+=item It won't work for compile-time warnings
 
-The best I can say about its method of messing with the source code of
-imported modules is that at least its modifications shouldn't stack with
-other source filters, so the degree of weirdness and potential insanity
-should be reduced to a manageable level.
+It works by fiddling with the (global) %warnings::Bits variable, and that's
+fine for run-time warnings. But if you say e.g.
+
+ use warnings;
+ use experimental 'signatures';
+ no warnings::anywhere 'experimental::signatures';
+ use Moose; # or Moo, or Dancer2, or... or...
+
+that won't work, as when Moose (or Moo, or Dancer2 etc.) injects all warnings
+into your package, it turns everything back on, and warnings::everywhere
+can't thwart that.
+
+(Previous versions of warnings::everywhere I<tried> to thwart this by
+basically a source filter, but that proved untenable.)
+
+The solution is to remember that Moose, Moo, Dancer2 etc. only turn on
+these compile-time warnings I<for this particular package>, so just say e.g.
+
+ use warnings;
+ use Moose; # or Moo etc.
+ use experimental 'signatures';
 
 =back
 
@@ -391,8 +337,7 @@ sub _set_bit_mask {
 sub _is_bit_set {
     my ($mask, $bit_num) = @_;
 
-    my $smallest_bit_num = $bit_num % 8;
-    return vec($mask, int($bit_num / 8), 8) & (1 << $smallest_bit_num);
+    return vec($mask, int($bit_num / 8), 8) & (1 << ($bit_num % 8));
 }
 
 sub _dump_mask {
@@ -417,96 +362,6 @@ sub _check_warning_category {
     return 1;
 }
 
-sub _thwart_modules {
-    my ($package, %args) = @_;
-
-    my @modules
-        = ref($args{thwart_module}) eq 'ARRAY'
-        ? @{ $args{thwart_module} }
-        : $args{thwart_module};
-
-
-    for my $module (@modules) {
-        $package->_thwart_this_module(%args, thwart_module => $module);
-    }
-}
-
-sub _thwart_this_module {
-    my ($package, %args) = @_;
-
-    # Moose's import sub is actually defined in Moose::Exporter, so
-    # modify that code instead.
-    my $module = $args{thwart_module};
-    if ($module eq 'Moose') {
-        $module = 'Moose::Exporter';
-    }
-    my $filename = $module;
-    $filename =~ s{::}{/}g;
-    $filename .= '.pm';
-    unshift @INC, sub {
-        my ($this_coderef, $use_filename) = @_;
-        return if $use_filename ne $filename;
-
-        # Find the source of the module we're looking for.
-        # This will fail if the module is itself being loaded by a
-        # coderef in @INC, say, but should work for the vast, vast
-        # majority of cases.
-        my $source_fh = $package->_find_module_source($use_filename)
-            or do {
-            croak "You asked me to thwart $args{thwart_module}"
-                . " but I can't find $use_filename anywhere in @INC";
-            };
-        my $source;
-        {
-            local $/;
-            $source = <$source_fh>;
-        }
-
-        # Work out what we're going to inject into this source code.
-        my @warnings
-            = ref($args{warning} eq 'ARRAY')
-            ? @{ $args{warning} }
-            : $args{warning};
-        my $source_code_unimport = "### Code injected by $package\n";
-        for my $warning (@warnings) {
-            $source_code_unimport .= qq{warnings->unimport("$warning");\n};
-        }
-        $source_code_unimport .= "### End of code injected by $package\n";
-
-        # Add this stuff just after a call to warnings->import (Moose, Moo,
-        # Dancer) or import::into (Dancer2, Test::Class::Moose).
-        my $re_code;
-        if ($module eq 'Dancer2') {
-            $re_code = qr/ ( import::into [^\n]+ warnings [^\n]+ ; \n ) /x;
-        } elsif ($module eq 'Test::Class::Moose') {
-            $re_code = qr/ ( \$_ -> import::into [^;\n]+ ; \n )/x;
-        } else {
-            $re_code = qr/ ( warnings->import; \n) /x;
-        }
-        $source =~ s/$re_code/$1$source_code_unimport/xsm
-            or croak
-            "Couldn't find a call to $re_code in $use_filename for $module\n$source\n";
-
-        # Right, return this modified source code.
-        open (my $fh_source, '<', \$source);
-        return $fh_source;
-    };
-}
-
-sub _find_module_source {
-    my ($package, $use_filename) = @_;
-
-    for my $dir (grep { !ref($_) } @INC) {
-        my $full_path = File::Spec->catfile($dir, $use_filename);
-        if (-e $full_path) {
-            open (my $fh, '<', $full_path);
-            return $fh;
-        }
-    }
-    return;
-}
-
-
 =back
 
 =head1 TO DO
